
-- Developed by Harsh Nagwanshi

require "import"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "android.content.Intent"

local editText = service.getEditText()
if not editText then
  service.asyncSpeak("Text box not found")
  return
end

-- Function to start actual speech typing with given language code
local function startTyping(languageCode)
  local speechRec = SpeechRecognizer.createSpeechRecognizer(service)
  local listener = RecognitionListener {
    onResults = function(results)
      local resultList = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
      if resultList and resultList.size() > 0 then
        local text = resultList.get(0)

        -- Smart replacements (Emojis, symbols)
        local replace = {
          ["thumbs up"] = "👍",
          ["red heart"] = "❤",
          ["folded hands"] = "🙏",
          ["new line"] = "\n",
          ["dot dot"] = "...",
        }

        for k, v in pairs(replace) do
          text = text:gsub(k, v)
        end

        if not text:match("[.?!\n]+$") then
          text = text .. ".\n"
        end

        service.insertText(editText, text)
        service.speak(text)
      end
      speechRec.destroy()
    end,
    onError = function()
      service.asyncSpeak("Something went wrong.")
      speechRec.destroy()
    end
  }

  local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
  intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
  intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, languageCode)

  speechRec.setRecognitionListener(listener)
  speechRec.startListening(intent)
end

-- First step: Ask for language in English
service.speak("Please say your preferred language: Hindi or English")

-- Listen to language choice
local langRecognizer = SpeechRecognizer.createSpeechRecognizer(service)
local langListener = RecognitionListener {
  onResults = function(results)
    local resultList = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
    if resultList and resultList.size() > 0 then
      local answer = resultList.get(0):lower()
      if answer:find("hindi") then
        startTyping("hi-IN")
      elseif answer:find("english") then
        startTyping("en-IN")
      else
        service.asyncSpeak("Language not recognized. Please say Hindi or English.")
      end
    end
    langRecognizer.destroy()
  end,
  onError = function()
    service.asyncSpeak("Could not understand your language choice.")
    langRecognizer.destroy()
  end
}

local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "en-IN")

langRecognizer.setRecognitionListener(langListener)
langRecognizer.startListening(intent)

